/*----------------------------------------------------------------------------
 * Titel     : Minimalbeispiel fuer INSN-Fehler in GCC 4.8.1
 *----------------------------------------------------------------------------
 * Hardware  : AVR ATmega32
 * Takt      : 16 MHz
 * Sprache   : AVR-GCC
 * Datum     : 30.07.2015
 * Version   : -
 * Autor     : Nicolas
 *----------------------------------------------------------------------------*/


#include <stdio.h>
#include "main.h"



// Puffer fuer Textfunktionen
#define N_TEXTBUF 81
char glstr_Buf[N_TEXTBUF];        // String fuer Textoutput

// Dummy fuer beliebige, ueber das Menue startbare Funktion
void dummy_fcn1(void) { }

// Dummy fuer beliebige, ueber das Menue manipulierpare Variable	
int16_t z_max = 0;

/* ************************************************************************** */
/*     Menue-Datensatz                                                        */
/* ************************************************************************** */

// Menue-Texte (Dummy)
constflash char MT_NULL[]     = {'\0'};
constflash char MT_RETURN[]   = "zurck";
constflash char MT_FCN1[]     = "Run Fcn1";
constflash char MT_VAR1[]     = "Z : %3imm";


static constflash menudata_t Menu_hlpfcn_dat[] =
{
    {MT_RETURN        ,voidFcn,{.pvoid=NULL}                   ,0,0},
    {MT_VAR1          ,int16  ,{.pint16 = &z_max}              ,0,100},		
    {MT_FCN1          ,voidFcn,{.pvoidFcn=dummy_fcn1}          ,0,0},
    {MT_NULL          ,nothing,{.pvoid=NULL}                   ,0,0}, // Nullzeile
};


#if (__GNUC__ == 4 && __GNUC_MINOR__ == 8 && __GNUC_PATCHLEVEL__ == 1)
    #warning "GCC 4.8.1 needs workaround for flash strings"

	/* WORKAROUND:
	   Unter AVR-GCC 4.8.1 sorgt die direkte Verwendung von
       "menudata[item].menutext" mit "snprintf_P" fuer einen Compilerfehler 
       ("unrecognizable insn"). 
	   
       Als Workaround wird der String zuerst ins SRAM kopiert und anschliessend 
       die snprintf-Funktion genutzt. 
	   
	   Fuer die Reproduktion der Fehlermeldung einfach die folgendende 
	   Definition auskommentieren: */
	   //#define WORKAROUND_STRING // <--- hier auskommentieren!
#endif


/* Hier ist der Ort der Fehlermeldung */
void draw_menutext_dummy(constflash menudata_t menudata[])
{
	uint_fast8_t item = 1;
	
	#ifdef WORKAROUND_STRING
		char tempstr[N_TEXTBUF];
		strncpy_P(tempstr,menudata[item].menutext,N_TEXTBUF);
		tempstr[N_TEXTBUF-1] = '\0';
		snprintf(glstr_Buf,N_TEXTBUF,tempstr,*menudata[item].pint16);
	#else
		snprintf_P(glstr_Buf,N_TEXTBUF,menudata[item].menutext,\
			*menudata[item].pint16);
	#endif
}


/* ************************************************************************** */
/*     Main                                                                   */
/* ************************************************************************** */
int main(void)
{
	draw_menutext_dummy(Menu_hlpfcn_dat);
  
    while(1) { } 
    
    return 0;
} 


